#!/bin/sh

# \
  if (test -z "$WISH"); then export WISH=wish; fi
# \
  exec "$WISH" "$0" -name wish ${1+"$@"}

# ==================================================================== #

# popup/examples/combobox.tcl --

# An example of a very basic combobox using the popup widget

# Version   : 0.0.1
# Author    : Mark G. Saye
# Email     : markgsaye@yahoo.com
# Copyright : Copyright (C) 2004
# Date      : March 17, 2004

# See the file "LICENSE.txt" or "LICENSE.html" for information on usage
# and distribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# ==================================================================== #

  namespace eval combobox {
    image create bitmap ::combobox::down -data {
      #define down_width 7
      #define down_height 4
      static unsigned char down_bits[] = {
        0x7f, 0x3e, 0x1c, 0x08 };
    }
    image create bitmap ::combobox::down -data {
      #define down_width 16
      #define down_height 16
      static unsigned char down_bits[] = {
         0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
         0x00, 0x00, 0x00, 0x00, 0xf0, 0x0f, 0xe0, 0x07,
         0xc0, 0x03, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00,
         0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
    }
  }

# ==================================================================== #

proc combobox::add {W} {

  set value [$W.entry get]
  if { ![string length $value] } { return }
  set L $W.popup.listbox

  # See if there's already an existing value in listbox
  set index -1
  if { [catch {$L cget -listvar} listvar] ||
      ![string length $listvar] } {
    # Search listbox for matching value
    for {set i 0} {$i < [$L size]} {incr i} {
      if { [string equal [$L get $i] $value] } {
        set index $i
      }
    }
  } else {
    upvar #0 $listvar list
    set index [lsearch $list $value]
  }

  if { $index == -1 } {
#   insert $W end $value
    $L insert end $value
    set index end
  }

  # Select matching item in listbox
  $W.popup.listbox activate $index
  $W.popup.listbox selection clear 0 end
  $W.popup.listbox selection set $index

}

# ==================================================================== #

proc combobox::create {W args} {

  array set arg [list \
    -borderwidth 1 \
    -highlightthickness 1 \
    -relief sunken \
  ]
  array set arg $args
  set arg(-class) Combobox

  array set _entry [list \
    -borderwidth 0 \
    -highlightthickness 0 \
  ]

  foreach option [list -textvariable] {
    if { [info exists arg($option)] } {
      set _entry($option) $arg($option)
      unset arg($option)
    }
  }

  eval [linsert [array get arg] 0 frame $W]

  eval [linsert [array get _entry] 0 entry $W.entry]

  label $W.down -image ::combobox::down -relief raised -bd 1 -padx 4 -pady 0

  popup $W.popup \
    -focus $W.popup.listbox \
    -borderwidth 0

  listbox $W.popup.listbox \
    -exportselection 0 \
    -height 0 \
    -selectmode browse

  pack propagate $W.popup 1

  pack $W.down          -side right -expand 0 -fill y
  pack $W.entry         -side left  -expand 1 -fill x
  pack $W.popup.listbox -side top   -expand 1 -fill both

  bind $W.down <ButtonPress-1> "[list combobox::map $W] ; break"
  bind $W.popup.listbox <ButtonRelease-1> [list combobox::select $W]
  bind $W.popup.listbox <Key-Return> [list combobox::select $W]
  bind $W.entry <Key-Down> [list combobox::map $W]
  bind $W.entry <Key-Return> [list combobox::add $W]
}

# ==================================================================== #

proc combobox::map {W} {
  set X [winfo rootx $W]
  set Y [expr {[winfo rooty $W] + [winfo height $W]}]
  $W.popup post $X $Y -width [winfo width $W]
}

# ==================================================================== #

proc combobox::insert {W index args} {
  eval [linsert $args 0 $W.popup.listbox insert $index]
}

# ==================================================================== #

proc combobox::select {W} {
  set sel [$W.popup.listbox curselection]
  if { [llength $sel] } {
    set index [lindex $sel 0]
    $W.entry delete 0 end
    $W.entry insert end [$W.popup.listbox get $index]
  }
  $W.popup unmap
}

# ==================================================================== #

  package require popup
  namespace import ::popup::*
  combobox::create .c
  combobox::insert .c end "item 1" "item 2" "item 3" "item 4"
  pack .c -fill x

# ==================================================================== #

