# popup/tcl/main.tcl --

# A popup widget.

# Version   : 0.0.1
# Author    : Mark G. Saye
# Email     : markgsaye@yahoo.com
# Copyright : Copyright (C) 2004
# Date      : March 17, 2004

# See the file "LICENSE.txt" or "LICENSE.html" for information on usage
# and distribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# ==================================================================== #

  namespace eval popup {}

# ==================================================================== #

# init_ --

# Description
#   Initialize

proc popup::init_ {} {

# -------------------------------------------------------------------- #

  variable class Popup

# -------------------------------------------------------------------- #

  init:bindings_
  init:options_

# -------------------------------------------------------------------- #

  return

}

# ==================================================================== #

# cmd_ --

# Description
#   Widget command

proc popup::cmd_ {W option args} {

  debug 2 "proc cmd_ \[$W\] \[$option\] \[$args\]\n"

  set error [catch [linsert $args 0 popup $option $W] return]

# -------------------------------------------------------------------- #

  if { $error } {
    set errorCode $::errorCode
    debug 3 "  error in proc cmd_\n"
    debug 3 "  error     = \[$error\]\n"
    debug 3 "  return    = \[$return\]\n"
    debug 3 "  errorCode = \[$errorCode\]\n"
    set code error
  } else {
    set code ok
  }

# -------------------------------------------------------------------- #

  debug 3 "  cmd_ return -code \[$code\] \[$return\]\n"
  return -code $code $return

}

# ==================================================================== #

# create --

# Description
#   Create a new popup widget

# Arguments
#   W    : popup widget name
#   args : arguments to be passed to the popup widget

# Return
#   the popup widget name

proc popup::create {W args} {

  debug 2 "proc create \[$W\] \[$args\]\n"

# -------------------------------------------------------------------- #

  variable class
  variable create
  variable defaults
  variable options
  variable options_toplevel
  variable require_toplevel

  upvar #0 [.]::$W _

debug 3 "  options = \[$options\]\n"
debug 3 "  options_toplevel = \[$options_toplevel\]\n"
debug 3 "  defaults = \[$defaults\]\n"
debug 3 "  create = \[$create\]\n"

# -------------------------------------------------------------------- #

  # set some default widget values
  set _(-focus) $W
  set _(-grab)  $W

# debug 3 array _

  array set _toplevel {}

# -------------------------------------------------------------------- #

  foreach option $create {
    set i [lsearch $options $option]
    set def [lindex $defaults $i]
    debug 3 "  set opt($option) \[$def\]\n"
    set opt($option) $def
  }

# -------------------------------------------------------------------- #

  foreach option $require_toplevel {
    set i [lsearch $options $option]
    set value [lindex $defaults $i]
    debug 3 "  import required toplevel option \[$option\] \[$value\]\n"
    set _toplevel($option) $value
  }

# -------------------------------------------------------------------- #

  foreach {option value} $args {
    if { [info exists map($option)] } {
      set option [lindex $map($option) 1]
    }

    set i [lsearch $options $option]

    if { $i > -1 } {
      # allowable option
      if { [lsearch $options_toplevel $option] > -1 } {
        set _toplevel($option) $value
      } else {
        set arg($option) $value
      }
    }
  }

# -------------------------------------------------------------------- #

  array set opt [array get arg]

# -------------------------------------------------------------------- #

  # create toplevel
# array set opt2 \
#   [eval [linsert [array get _toplevel] 0 create:toplevel_ $W]]
# parray _toplevel
  debug 3 "  toplevel \[$W\] \[[array get _toplevel]\]\n"
  eval [linsert [array get _toplevel] 0 toplevel $W]

  wm withdraw $W
  update idletasks

  wm overrideredirect $W 1

# -------------------------------------------------------------------- #

  # import option database
# array set opt [${ew}::widget::options:import $W]
# array set dbopt [${ew}::widget::options:import $W]
# foreach {option value} [array get dbopt] {
#   if { [info exists options_toplevel($option)] } {
#     # option already auto-imported
#     debug 3 "  option \[$option\] = \[$value\] already imported\n"
#   } else {
#     set opt2($option) $value
#   }
# }

# -------------------------------------------------------------------- #

  # args override opts
# array set opt2 [array get arg]

# -------------------------------------------------------------------- #

  # move widget command into our namespace
  rename $W $W

  # alias global command to cmd_ proc
  interp alias {} ::$W {} [.]::cmd_ $W

# -------------------------------------------------------------------- #

  # configure widget
  set error 0
  set error [catch [linsert [array get arg] 0 configure $W] result]

  if { $error } {
    set errorCode $::errorCode
    debug 3 "  error     = \[$error\]\n"
    debug 3 "  errorCode = \[$::errorCode\]\n"
    destroy $W
    return -code error $error
  }

# -------------------------------------------------------------------- #

  # bindings
  bindtags $W [list $W $class bind$W all]

# bind $W <Tab>           "break"
# bind $W <Shift-Tab>     "break"
# bind $W <<PrevWindow>>  "break"
# bind $W <Escape>        "[list [.]::unmap $W] ; break"

# bind $W <ButtonPress-1> [list [.]::press1_ $W %X %Y]

  # bindings for popup and all children
  bind $W <Escape>        "[list [.]::unmap $W] ; break"

# -------------------------------------------------------------------- #

  # return the name of the widget
  debug 3 "  create \[$W\] return \[$W\]\n"
  return $W

}

# ==================================================================== #

# create:toplevel_ --

# Description
#   Create the toplevel widget

# Arguments
#   W : toplevel widget
#   args : arguments for toplevel widget

# Return
#   the toplevel widget name

proc popup::create:toplevel_ {W args} {

  debug 2 "proc create:toplevel_ \[$W\] \[$args\]\n"

# -------------------------------------------------------------------- #

  variable  create_toplevel
  variable options_toplevel
  variable require_toplevel

  upvar #0 [.]::$W _

# -------------------------------------------------------------------- #

# array set arg [array get options_toplevel]
  foreach option $require_toplevel {
    set arg($option) $options_toplevel($option)
  }

# -------------------------------------------------------------------- #

# array set arg $args
  foreach {option value} $args {
    if { [info exists options_toplevel($option)] } {
      set arg($option) $value
    } else {
      set opt($option) $value
    }
  }

# -------------------------------------------------------------------- #

  # unset some empty options disliked by toplevel
  foreach option {-colormap -use} {
    if { [info exists arg($option)] &&
        ![string length $arg($option)] } {
      unset arg($option)
    }
  }

# -------------------------------------------------------------------- #

  # create toplevel
  debug 3 "  toplevel [list $W] [array get arg]\n"
  eval [linsert [array get arg] 0 toplevel $W]

  wm withdraw $W
  wm overrideredirect $W 1
  update idletasks

# -------------------------------------------------------------------- #

  # return list of options not used by toplevel
  debug 3 "  create:toplevel_ return \[[array get opt]\]\n"
  return [array get opt]

}

# ==================================================================== #

# init:bindings_ --

# Description
#   Initialize class bindings

proc popup::init:bindings_ {} {

  debug 2 "proc init:bindings_\n"

# -------------------------------------------------------------------- #

  variable class

# -------------------------------------------------------------------- #

  bind $class <Map>   [list [.]::map   %W -i %i -o %o -E %E -T %T]
  bind $class <Unmap> [list [.]::unmap %W -i %i -o %o -E %E -T %T]
# bind $class <Tab>           "break"
# bind $class <Shift-Tab>     "break"
# bind $class <<PrevWindow>>  "break"
  bind $class <Destroy>        [list [.]::destroy %W]
  bind $class <Escape>        "[list [.]::unmap %W] ; break"

  bind $class <ButtonPress-1> [list [.]::press1_ %W %X %Y]

# -------------------------------------------------------------------- #

  return

}

# ==================================================================== #

# destroy --

# Description
#   Destroy a popup widget

proc popup::destroy {W} {

  debug 2 "proc destroy \[$W\]\n"

# -------------------------------------------------------------------- #

# variable $W
  upvar #0 [.]::$W _

# -------------------------------------------------------------------- #

  if { [winfo exists $W] } {
    bindtags $W [list]
    ::destroy $W
  }

  array unset _

# -------------------------------------------------------------------- #

  return

}

# ==================================================================== #

# map --

# Description
#   Display the popup/menu window

proc popup::map {W args} {

  debug 2 "proc map \[$W\] \[$args\]\n"

# -------------------------------------------------------------------- #

  variable class

  upvar #0 [.]::$W _

# -------------------------------------------------------------------- #

  if { [winfo ismapped $W] } {
    set _(focus) [focus -displayof $W]

#   event generate $_(focus) <Leave>

    debug 3 "  focus $_(-focus)\n"
    focus $_(-focus)

    debug 3 "  grab set -global \[p$_(-grab)\]\n"
    if { [string length $_(-grab)] } {
      grab set -global $_(-grab)
    }
  } else {
    set x [winfo rootx $W]
    set y [winfo rooty $W]
    post $W $x $y
  }

# -------------------------------------------------------------------- #

  return

}

# ==================================================================== #

# popup --

# Description
#   Main widget command

# Arguments
#   command : widget command
#   args    : arguments to be passed to widget command

# Return
#   return value from widget command

proc popup::popup {command args} {

  debug 2 "proc popup \[$command\] \[$args\]\n"

# -------------------------------------------------------------------- #

  if { [string match .* $command] } {
    set args [linsert $args 0 $command]
    set command create
  }

# -------------------------------------------------------------------- #

  set which [namespace which -command $command]
  set path  [namespace qualifiers $which]

  if { [string equal $path [.]] } {
    set error [catch [linsert $args 0 $command] return]
  } else {
    set procs [list]
    foreach proc [lsort [info procs]] {
      if { [string match *_ $proc] } { continue }
      lappend procs $proc
    }
    set must [join [lrange $procs 0 end-1] ", "]
    append must " or [lindex $procs end]"
    set error 1
    set return "unknown command \"$command\": must be one of $must"
  }

# -------------------------------------------------------------------- #

  if { $error } {
    set errorCode $::errorCode
    set errorInfo $::errorInfo
    debug 3 "  error in proc popup & $command\n"
    debug 3 "  error     = \[$error\]\n"
    debug 3 "  return    = \[$return\]\n"
    debug 3 "  errorCode = \[$errorCode\]\n"
    set code error
  } else {
    set code ok
  }

# -------------------------------------------------------------------- #

  debug 3 "  popup return -code \[$code\] \[$return\]\n"
  return -code $code $return

}

# ==================================================================== #

# post --

# Description
#   Post/display the popup window

proc popup::post {W X Y args} {

  debug 2 "proc post \[$W\] \[$X\] \[$Y\] \[$args\]\n"

# -------------------------------------------------------------------- #

  variable class

  upvar #0 [.]::$W _

# -------------------------------------------------------------------- #

  array set arg [list \
    -shiftx 1 \
    -shifty 0 \
    -width  [winfo reqwidth  $W] \
    -height [winfo reqheight $W] \
  ]

  foreach {option value} $args {
    if { [info exists arg($option)] } {
      set arg($option) $value
    } else {
      return -code error "unknown option \"$option\""
    }
  }

# -------------------------------------------------------------------- #

  set h 200
  set w  $arg(-width)
  set h  $arg(-height)
  set sw [winfo screenwidth  $W]
  set sh [winfo screenheight $W]


# -------------------------------------------------------------------- #

  # Handle negative offset from right side of screen
# if { $X < 0 } { set X [expr {0 - $X - $w}] }
# debug 3 "  $W configure -width $w -height $h\n"

  if { $X < 0 } { set X1 0 } { set X1 $X }
  if { $Y < 0 } { set Y1 0 } { set Y1 $Y }

  set X2 [expr {$X1 + $w}]
  set Y2 [expr {$Y1 + $h}]

  if { $X2 > $sw } {
    set X2 $sw
    if { $arg(-shiftx) } {
      set X1 [expr {$sw - $w}]
      if { $X1 < 0 } { set X1 0 }
    }
  }

  if { $Y2 > $sh } {
    set Y2 $sh
    if { $arg(-shifty) } {
      set Y1 [expr {$sh - $h}]
      if { $Y1 < 0 } { set Y1 0 }
    }
  }

  set w [expr {$X2 - $X1}]
  set h [expr {$Y2 - $Y1}]

# -------------------------------------------------------------------- #

# set _class [winfo class $W]
# debug 3 "  _class = \[$_class\]\n"
# if { [string equal $_class $class] } {

#   $W configure -width $w -height $h
#   wm geometry  $W.top ${w}x${h}+${x1}+${Y1}
    debug 3 "  wm geometry  $W +${X1}+${Y1}\n"
    wm geometry  $W ${w}x${h}+${X1}+${Y1}
    debug 3 "  wm deiconify $W\n"
    wm deiconify $W
    tkwait visibility $W
    raise $W

# } elseif { [string equal $_class Menu] } {
#   bind $W <Map> [list +puts "Map $W"]
#   $W post $X $y
# }

# -------------------------------------------------------------------- #

  debug 3 "  winfo width  $W = \[[winfo width  $W]\]\n"
  debug 3 "  winfo height $W = \[[winfo height $W]\]\n"

# -------------------------------------------------------------------- #

  return

}

# ==================================================================== #

proc popup::press1_ {W X Y} {

  debug 2 "proc press1_ \[$W\] \[$X\] \[$Y\]\n"

# -------------------------------------------------------------------- #

  upvar #0 [.]::$W _

# -------------------------------------------------------------------- #

  set w  [winfo width  $W]
  set h  [winfo height $W]
  set X1 [winfo rootx  $W]
  set Y1 [winfo rooty  $W]
  set X2 [expr $X1 + $w]
  set Y2 [expr $Y1 + $h]

  if { $X < $X1 || $X > $X2 || $Y < $Y1 || $Y > $Y2 } { unmap $W }

# -------------------------------------------------------------------- #

  return

}

# ==================================================================== #

# unmap --

# Description
#   Unmap the popup window from the display

# Arguments
#   W    : popup window
#   args : arguments

proc popup::unmap {W args} {

  debug 2 "proc unmap \[$W\] \[$args\]\n"

# -------------------------------------------------------------------- #

  upvar #0 [.]::$W _

# -------------------------------------------------------------------- #

  debug 3 "  grab release $_(-grab)\n"
  grab release $_(-grab)

# -------------------------------------------------------------------- #

  if { [info exists _(focus)] } {
    if { [catch {::focus -force $_(focus)} error] } {
      debug 0 "  \[$::errorCode\] $error\n"
    } else {
      debug 3 "  restored focus to '$_(focus)'\n"
    }
    unset _(focus)
  }

# -------------------------------------------------------------------- #

  if { [winfo ismapped $W] } {
    debug 3 "  wm withdraw $W\n"
    wm withdraw $W
  }

# -------------------------------------------------------------------- #

  return

}

# ==================================================================== #

# unpost --

proc popup::unpost {W} {

  debug 2 "proc unpost \[$W\]\n"

# -------------------------------------------------------------------- #

  if { [winfo ismapped $W] } {
    debug 3 "  wm withdraw $W\n"
    wm withdraw $W
  }

# -------------------------------------------------------------------- #

  return

}

# ==================================================================== #

  popup::init_

  return

# ==================================================================== #

