# popup.tcl --

# A popup widget.

# Version   : 0.0.1
# Author    : Mark G. Saye
# Email     : markgsaye@yahoo.com
# Copyright : Copyright (C) 2004
# Date      : March 17, 2004

# See the file "LICENSE.txt" or "LICENSE.html" for information on usage
# and distribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.

# ==================================================================== #

  namespace eval popup {}

# ==================================================================== #

# pkginit_ --

# Description
#   Initialize this package

proc popup::pkginit_ {} {

# -------------------------------------------------------------------- #

  interp alias {} [::namespace current]::.  {} ::namespace current

# -------------------------------------------------------------------- #

  if { [catch {package require debug} version] } {
proc debug {level string} {
  variable debug
  set i [uplevel [list info level]]
  if { $level <= $debug } {
    puts -nonewline stderr "\[$i\] popup: $string"
  }
  return
}
  }

  variable debug ; if { ![info exists debug] } { set debug 1 }

  debug 2 " proc pkginit_\n"

# -------------------------------------------------------------------- #

  if { [catch {file normalize [info script]} scr] } {
    set scr [file join [pwd] [info script]]
  }
  set tcl [file dirname $scr]
  set dir [file dirname $tcl]

  ::variable library $dir

  debug 3 "  script = \[$scr\]\n"

# -------------------------------------------------------------------- #

  variable pkginfo ; array set pkginfo [list \
    package      popup \
    version      0.0.1 \
    script       $scr \
    directory    $dir \
    name        "Popup Widget" \
    description "A popup widget." \
    copyright   "Copyright (C) 2004" \
    author      "Mark G. Saye" \
    email       "markgsaye@yahoo.com" \
    date        "March 17, 2004" \
  ]

# -------------------------------------------------------------------- #

  set dir  [file join $dir tcl]
  set file [file join $dir tclIndex]

  if { ![file isfile $file] || ![catch {source $file} error] } {
    foreach command [lsort [array names auto_index]] {
      regsub -- [.]:: $command "" cmd
      if { [string equal $cmd debug] } { continue }
      if { [string equal $cmd load_] } { continue }
#     set cmd [namespace tail $command]
#     debug 5 "  set ::auto_index([.]::$cmd) \[$auto_index($command)\]\n"
#     set ::auto_index([.]::$cmd) $auto_index($command)
      set file [lindex $auto_index($command) 1]
      set name [file rootname [file tail $file]]
      debug 3 "  alias \[[.]::$cmd\] \[[.]::load_\] \[$name\] \[$cmd\]\n"
      interp alias {} [.]::$cmd {} [.]::load_ $name $cmd
    }
  }

# -------------------------------------------------------------------- #

  namespace export popup

  if { [string equal ::$pkginfo(package) [.]] } {
    debug 3 "  package provide \[$pkginfo(package)\] \[$pkginfo(version)\]\n"
    package provide $pkginfo(package) $pkginfo(version)
  }

# -------------------------------------------------------------------- #

  rename pkginit_ {}

# -------------------------------------------------------------------- #

  return

}

# ======================================================================

# load_ --

# Description
#   Load package routines and call package command

# Arguments
#   name    : name of package routines file
#   command : package command to call
#   args    : arguments for package command

# Return
#   Result from package command

proc popup::load_ {name command args} {

  debug 2 "proc load_ \[$name\] \[$command\] \[$args\]\n"

# ----------------------------------------------------------------------

  variable pkginfo
  variable library

# ----------------------------------------------------------------------

  set file [file join $library tcl ${name}.tcl]

  debug 3 "  source \[$file\]\n"
  set result [namespace eval [namespace parent] [list source $file]]
  debug 3 "  source \[$file\] -> \[$result\]\n"

# ----------------------------------------------------------------------

  set error [catch {uplevel [linsert $args 0 [.]::$command]} return]

  if { $error } { set code error } { set code ok }

# ----------------------------------------------------------------------

  debug 2 "  load return -code \[$code\] \[$return\]\n"
  return -code $code $return

}

# ======================================================================

# pkginfo --

# Description
#   Return the package info

proc popup::pkginfo {} {

  debug 2 "proc pkginfo\n"

# ----------------------------------------------------------------------

  variable pkginfo

# ----------------------------------------------------------------------

  return [array get pkginfo]

}

# ==================================================================== #

  popup::pkginit_

  return

# ==================================================================== #

